/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.util.Random;

/**
 * This class implements an (unnormalized) Exponential probability density,
 * truncated so as to live on the interval (0,1).
 * @author L.A. Breyer
 * @version 1.0
 */
final class Exponential extends Pdf {

    double lambda;

	/**
	 * This constructor creates an exponential density with inverse
	 * mean l, truncated on the interval (0,1).
	 * @param l the inverse mean
	 */
    Exponential(double l) {
	lambda = l;
    }

    	/**
	 * This method returns an Exponential (inverse mean lambda)
	 * random variable. Note this is *not* truncated to (0,1).
	 * @param r a random number generator
	 * @return an exponential random variable
	 */
    public double newVariable(Random r) {
	return -Math.log(r.nextDouble())/lambda;
    }
 
    public double valueAt(double x) {
	if( (x < 0) || (x > 1) )
	    return 0;
	else
	    return Math.exp( - lambda * x );
    }

    public double densityRatio(double x, double y) {
	if( (x < 0) || (x > 1) )
	    return 0;
	else
	    return Math.exp( lambda *(x - y));
    }
    
    public RealInterval getDensityRange() {
	return new RealInterval(0,1);
    }

    public double gradLogAt(double x) {
	return (-lambda);
    }

    public RealInterval gradLogRange(RealInterval r) {
	return new RealInterval(-lambda, -lambda);
    }

    public RealInterval range(RealInterval xrange) {
	return new RealInterval( Math.exp( -lambda * xrange.end), Math.exp( -lambda * xrange.beginning) );
    }
    
    public double getMean() {
	return 1/lambda;
    }

    public double getVariance() {
	return 1/(lambda * lambda);
    }
}
