/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.io.*;

/**
 * This class implements a frame in which a variety of classical
 * Metropolis-Hastings algorithms can be coupled.
 * 
 * @author Laird A. Breyer
 */
class FCFrame extends Frame implements Runnable, Observer, WindowListener {

    MyRandom r;
    Pdf pdf;
    int algo;
    RandomField rx, rx2;
    CouplableField x, gx;
    CoupledGaussianField cgx, cgxstack[];

//    HybridMonteCarloField hx;
//    double chain, hybridChain[], pdfMax;
    
    double rwmSigma;
    double lanSigma;
    double gorSigma;
    double garA, garB, garSigma;
    double lfmSigma, lfmAlpha;
    double trwSigma, trwNu;
    double hamAlpha, hamEps;
    int hamL;
    Pdf uuiPdf;

    double h, Mag, z, y, yold, oldval, newval, oldtargetval, newtargetval;
    String msg;
    Thread appletThread;
    long delay;
    int numbins, mynumbins;
    boolean bins[];
    long histDelay;
    int histcount, histType = 3, histGridSize = 50;
    int demoType = 1;
    boolean resetFlag = false;
    boolean cleared_histogram = false;

//    int acfMaxLag, histNBins;

    PdfDisplayPanel pdfDisplayPanel;
    PdfControlPanel pdfControlPanel;
    
    CoupledFieldGraphPanel coupledFieldGraphPanel;
    AlgoControlPanel algoControlPanel;

    FieldHistogram fieldHistogram;
    TimeHistogram timeHistogram;
    FieldCouplerControlPanel fcControlPanel;

    FieldHistogram fieldDensityGraph;

    static boolean standalone = false;

    FCFrame() {
	super("Field Couplers");
	addWindowListener(this);

	setFont(new Font("Dialog",Font.PLAIN, 10));

	setLayout(new GridLayout(0,2, 0, 0)); 
	Panel p1 = new Panel(new GridLayout(4,0));
	Panel p2 = new Panel(new GridLayout(4,0));

	add(p1);
	add(p2);
	
	pdfDisplayPanel = new PdfDisplayPanel();
	Panel temp = new Panel(new GridLayout(2,0));
	temp.add(new TitlePanel("Field Couplers"));
	temp.add(pdfDisplayPanel);

	p1.add(temp);

	pdfControlPanel = new PdfControlPanel();
	pdfControlPanel.addObserver(this);

	p2.add(pdfControlPanel);
	
	coupledFieldGraphPanel = new CoupledFieldGraphPanel();

	p1.add(coupledFieldGraphPanel);
	
 	algoControlPanel = new AlgoControlPanel();
	algoControlPanel.addObserver(this);

	p2.add(algoControlPanel);

	fieldHistogram = new FieldHistogram(histGridSize, histType);
	fieldHistogram.set_title("coupled + MH");

	p1.add(fieldHistogram);
	
	fcControlPanel = new FieldCouplerControlPanel();
	fcControlPanel.addObserver(this);

	p2.add(fcControlPanel);

	fieldDensityGraph = new FieldHistogram(histGridSize, histType); //DensityGraph(histGridSize);
	fieldDensityGraph.set_title("uncoupled + MH");
	p1.add(fieldDensityGraph);

	timeHistogram = new TimeHistogram(550);

	p2.add(timeHistogram);
	
	r = new MyRandom();

    }

    public void init() {
	delay = (long)FieldCouplerControlPanel.DEFITDELAY;
	numbins = mynumbins = 8;
	histDelay = (long)FieldCouplerControlPanel.DEFHISTDELAY;
	histcount = 1;
	resetFlag = true;
	pdf = new Normal(PdfControlPanel.NORMALDEFMU, PdfControlPanel.NORMALDEFSIGMA2);
	pdfControlPanel.doInit();
	algo = AlgoControlPanel.RWM;
	rwmSigma = AlgoControlPanel.RWMDEFSIGMA;
	algoControlPanel.doInit();
	
	pdfDisplayPanel.clearDrawingBoard();
	fieldHistogram.reset(histGridSize, histType);
	fieldHistogram.repaint();

	fieldDensityGraph.reset(histGridSize, histType);
	fieldDensityGraph.repaint();

	timeHistogram.reset(550);
	timeHistogram.repaint();
	
	cgxstack = new CoupledGaussianField[500];
	resetCgxStack(numbins);
	cgx = new CoupledGaussianField();

	rx = new HMField(r, null, null);
	rx2 = new HMField(r, null, null);

// 	chain = 0.5;
// 	hybridChain = new double[2];
// 	hybridChain[0] = 0.5;
// 	hybridChain[1] = 1;
	
// 	delay = (long)DiagControlPanel.DEFITDELAY;
// 	acfMaxLag = (int)DiagControlPanel.DEFACFLAG;
// 	histNBins = (int)DiagControlPanel.DEFNHIST;
	
// 	pdf = new Normal(PdfControlPanel.NORMALDEFMU, PdfControlPanel.NORMALDEFSIGMA2);
// 	pdfMax = 1;
	
// 	pdfDisplayPanel.setPdf(pdf);
// 	pdfDisplayPanel.clearDrawingBoard();
// 	pdfDisplayPanel.showGraph();

// 	targetAverage.reset(histNBins);
// 	targetAverage.calibrate(pdf);
// 	acfPlot.reset(acfMaxLag, targetAverage.getTheoMean(), targetAverage.getTheoVar());

// 	algo = AlgoControlPanel.RWM;
// 	rwmSigma = AlgoControlPanel.RWMDEFSIGMA;

    }
    
    public void update(Observable obj, Object arg) {
	if(obj == pdfControlPanel.gopher) {
	    if( arg == null ) {
		pdf = null;
		pdfDisplayPanel.setPdf(pdf);
		pdfDisplayPanel.showGraph();
		pdfDisplayPanel.repaint();
	    } else {
		if( arg instanceof String ) {
		    if( ((String)arg).equals("New") ) {
			pdfDisplayPanel.clearDrawingBoard();
			pdfDisplayPanel.showDrawingBoard();
			pdf = null;
		    } else if( ((String)arg).equals("Accept") ) {
			pdf = pdfDisplayPanel.getPdf();
			pdfDisplayPanel.setPdf(pdf);
			pdfDisplayPanel.showGraph();
			pdfDisplayPanel.repaint();
//			targetAverage.reset(histNBins);
//			targetAverage.calibrate(pdf);

//			acfPlot.reset(acfMaxLag, targetAverage.getTheoMean(),
//				      targetAverage.getTheoVar());
		    } else if( ((String)arg).equals("Load") ) {
			FileDialog fd = new FileDialog(this, "Load a pdf", FileDialog.LOAD);
			fd.setFile("*.pdf");
			fd.setVisible(true);
			if( fd.getFile() != null ) {
			    String where = fd.getDirectory() + fd.getFile();
			    try {
				pdfDisplayPanel.loadPdf(where);
 				pdf = pdfDisplayPanel.getPdf();
 				pdfDisplayPanel.setPdf(pdf);
// 				targetAverage.reset(histNBins);
// 				targetAverage.calibrate(pdf);

//				acfPlot.reset(acfMaxLag, targetAverage.getTheoMean(),
//					      targetAverage.getTheoVar());
			    } catch( IOException ioe ) {
				pdf = null;
			    }
			}
			fd = null;
		    } else if( ((String)arg).equals("Save") ) {
			FileDialog fd = new FileDialog(this, "Save a pdf", FileDialog.SAVE);
			fd.setFile("noname.pdf");
			fd.setVisible(true);
			if( fd.getFile() != null ) {
			    String where = fd.getDirectory() + fd.getFile();
			    try {
			    pdfDisplayPanel.savePdf(where);
			    } catch( IOException ioe ) {
				// do nothing
			    }
			}			
			fd = null;
		    }
		} else if( arg instanceof Pdf ) {
		    pdf = (Pdf)arg;
		    pdfDisplayPanel.setPdf(pdf);
		    pdfDisplayPanel.showGraph();
		    pdfDisplayPanel.repaint();
//		    targetAverage.reset(histNBins);
//		    targetAverage.calibrate(pdf);

//		    acfPlot.reset(acfMaxLag, targetAverage.getTheoMean(),
//				  targetAverage.getTheoVar());
		    fieldHistogram.reset(histGridSize, histType);
		    fieldHistogram.repaint();

		    fieldDensityGraph.reset(histGridSize, histType);
		    fieldDensityGraph.repaint();
		    
		    timeHistogram.reset(550);
		    timeHistogram.repaint();
		    
		    histcount = 1;
		    resetFlag = true;

		}
//		if( pdf != null )
//		    pdfMax = pdf.getDensityRange().end;
//		else
//		    pdfMax = 1;
	    }
	} else if( obj == algoControlPanel.gopher ) {
	    cgx = null;
	    gx = null;
	    resetFlag = true;
	    if(arg == null) {
		algo = AlgoControlPanel.RWM;
		rwmSigma = AlgoControlPanel.RWMDEFSIGMA;
	    } else {
		algo = ((Integer)
			((Vector)arg).elementAt(0)).intValue();
		switch(algo) {
		    case AlgoControlPanel.RWM:
			rwmSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case AlgoControlPanel.LAN:
			lanSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case AlgoControlPanel.LFM:
			lfmSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			lfmAlpha = ((Double)
				    ((Vector)arg).elementAt(2)).doubleValue();
			break;
		    case AlgoControlPanel.TRW:
			trwSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			trwNu = ((Double)
				    ((Vector)arg).elementAt(2)).doubleValue();
			break;
		    case AlgoControlPanel.UI:
			uuiPdf = null;
			break;
		    case AlgoControlPanel.UUI:
			Object e = ((Vector)arg).elementAt(1);
			if( e == null ) {
			    uuiPdf = null;
			} else if( e instanceof Pdf ) {
			    uuiPdf = (Pdf) e;
			} else if( e instanceof String ) {
			    PdfDisplayPanel udp = (PdfDisplayPanel)((Vector)arg).elementAt(2);
			    if( ((String)e).equals("Load") ) {
				FileDialog fd = new FileDialog(this, "Load a proposal pdf",
							       FileDialog.LOAD);
				fd.setFile("*.pdf");
				fd.setVisible(true);
				if( fd.getFile() != null ) {
				    String where = fd.getDirectory() + fd.getFile();
				    try {
					udp.loadPdf(where);
					uuiPdf = udp.getPdf();
					udp.setPdf(uuiPdf);
				    } catch( IOException ioe ) {
					uuiPdf = null;
				    }
				}
				fd = null;
				udp.repaint();
			    } else if( ((String)e).equals("Save")) {
				FileDialog fd = new FileDialog(this, "Save a pdf", FileDialog.SAVE);
				fd.setFile("noname.pdf");
				fd.setVisible(true);
				if( fd.getFile() != null ) {
				    String where = fd.getDirectory() + fd.getFile();
				    try {
					udp.savePdf(where);
				    } catch( IOException ioe ) {
				// do nothing
				    }
				}			
				fd = null;
				udp.repaint();
			    }
			}
			break;
		    case AlgoControlPanel.GOR:
			gorSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case AlgoControlPanel.GAR:
			garA = ((Double)
				((Vector)arg).elementAt(1)).doubleValue();
			garB = ((Double)
				((Vector)arg).elementAt(2)).doubleValue();
			garSigma = ((Double)
				    ((Vector)arg).elementAt(3)).doubleValue();
			break;
		    case AlgoControlPanel.HYB:
			hamAlpha = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			hamEps = ((Double)
				  ((Vector)arg).elementAt(2)).doubleValue();
			hamL = ((Integer)
				((Vector)arg).elementAt(3)).intValue();
			break;
		    default:
		}
	    }
	    
	    fieldHistogram.reset(histGridSize, histType);
	    fieldHistogram.computeHistogram();
	    fieldHistogram.repaint();

	    fieldDensityGraph.reset(histGridSize, histType);
	    fieldDensityGraph.computeHistogram();
	    fieldDensityGraph.repaint();

	} else if( obj == fcControlPanel.gopher ) {
	    if( arg instanceof String ) {
		if( ((String)arg).equals(FieldCouplerControlPanel.RESETSTR)) {
		    fieldHistogram.reset(histGridSize, histType);
		    fieldHistogram.computeHistogram();
		    fieldHistogram.repaint();
		    histcount = 1;

		    fieldDensityGraph.reset(histGridSize, histType);
		    fieldDensityGraph.computeHistogram();
		    fieldDensityGraph.repaint();

		    //fieldDensityGraph.repaint();
		    timeHistogram.reset(550);
		    timeHistogram.repaint();
		    resetFlag = true;
		} else if( ((String)arg).equals(FieldCouplerControlPanel.CB1STR)) {
		    histType = 1;
		    fieldHistogram.setType(histType);
		} else if( ((String)arg).equals(FieldCouplerControlPanel.CB2STR)) {
		    histType = 2;
		    fieldHistogram.setType(histType);
		} else if( ((String)arg).equals(FieldCouplerControlPanel.CB3STR)) {
		    histType = 3;
		    fieldHistogram.setType(histType);
		} else if( ((String)arg).equals(FieldCouplerControlPanel.CCB1STR)) {
		    demoType = 1;
		} else if( ((String)arg).equals(FieldCouplerControlPanel.CCB2STR)) {
		    demoType = 2;
		} else if( ((String)arg).equals(FieldCouplerControlPanel.CCB3STR)) {
		    demoType = 3;
		} else if( ((String)arg).equals(FieldCouplerControlPanel.CCB4STR)) {
		    demoType = 4;
		}
	    } else if( arg instanceof ActionEvent ) {
		double dvalue = ((ParamInputField)
				 ((ActionEvent)arg).getSource()).getValue();
		String ac = ((ActionEvent)arg).getActionCommand();
		if( ac.equals(FieldCouplerControlPanel.ITDELAY) ) 
		    delay = (long)Math.round(dvalue);
		else if( ac.equals(FieldCouplerControlPanel.ITBTHIST) ) 
		    histDelay = (long)Math.round(dvalue);
		else {
		    mynumbins = (int)Math.round(dvalue);
		    resetFlag = true;
	        }
	    }
	}

	//fieldDensityGraph.setField(getCurrentField());
	//fieldDensityGraph.computeDensityGraph();
	//fieldDensityGraph.repaint();	

//	targetAverage.repaint();

	//	System.gc();	
    }

    CouplableField getCurrentField() {
	CouplableField f;

	if( pdf == null )
	    f = null;
	else {
	    switch(algo) {
		case AlgoControlPanel.RWM: 
		    f = new GaussianRandomWalkField(r, rwmSigma);
		    break;
		case AlgoControlPanel.LAN: 
		    f = new LangevinField(r, pdf, lanSigma);
		    break;
		case AlgoControlPanel.UI: 
		    f = new IndependenceField(r);
		    break;
		case AlgoControlPanel.GOR:
		    f = new GorField(r, pdf, gorSigma);
		    break;
		case AlgoControlPanel.GAR:
		    f = new GaussianARField(r, garA, garB, garSigma);
		    break;
		default:
		    f = null;
	    }
	}
	return f;
    }

    void resetCgxStack(int numbins) {
	for(int numfields = 0; numfields < 500; numfields++ ) {
	    cgxstack[numfields] = new CoupledGaussianField(numbins, null, null);
	}
    }

    
    public void start() {
	if( pdf == null ) {
		// do nothing
	} else {
	    if( appletThread == null ) {
		appletThread = new Thread(this);
	    }
	    appletThread.start();	
	    //} else if( !appletThread.isAlive() )
	    //	appletThread.start();	    
	}
    }

    public void stop() {
	appletThread = null;
    }
    
    public void run() {

	Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
	Thread currentThread = Thread.currentThread();
	
	while( currentThread == appletThread ) {

	    if( resetFlag ) {
		resetFlag = false;
		gx = cgx = null;
		rx = rx2 = null;
	    }

	    try {
		Thread.sleep(delay);
		Thread.sleep(10); // for Netscape
	    } catch( InterruptedException e) {};

	    if( pdf == null ) 	
		continue;

	    if( mynumbins != numbins )
		numbins = mynumbins;
		
	    // create the base random field
	    switch(algo) {
		case AlgoControlPanel.RWM:
		    if( gx == null )
			gx = new GaussianRandomWalkField(r, rwmSigma);
		    else
			//gx = new GaussianRandomWalkField(r, rwmSigma);
			gx = ((GaussianRandomWalkField)gx).respawn(r, rwmSigma);
		    break;
		case AlgoControlPanel.LAN:
		    if( gx == null )
			gx = new LangevinField(r, pdf, lanSigma);
		    else
			//gx = new LangevinField(r, pdf, lanSigma);
			gx = ((LangevinField)gx).respawn(r, pdf, lanSigma);
		    break;
		case AlgoControlPanel.GOR:
		    if( gx == null )
			gx = new GorField(r, pdf, gorSigma);
		    else
			//gx = new GorField(r, pdf, gorSigma);
			gx = ((GorField)gx).respawn(r, pdf, gorSigma);
		    break;
		default:
		    gx = null;
		    if( !cleared_histogram ) {
			fieldHistogram.reset(histGridSize, histType);
			fieldHistogram.computeHistogram();
			fieldHistogram.repaint();
			cleared_histogram = true;

			fieldDensityGraph.reset(histGridSize, histType);
			fieldDensityGraph.computeHistogram();
			fieldDensityGraph.repaint();
		    }
	    }

	    if( gx == null )
		continue;

	    // if we get through here, then we're ok
	    cleared_histogram = false; // for next time we choose a nono algorithm

	    int numfields = 0;

	    bins = new boolean[numbins];
	    boolean done = false;
	    // now build the list of coupled fields
	    for(numfields = 0; numfields < 500 && (!done) && (!resetFlag); numfields++ ) {
		//cgxstack[numfields] = new CoupledGaussianField(r,numbins,null,(GaussianEstimates)gx);
		cgxstack[numfields] = cgxstack[numfields].respawn(r, numbins, null, (GaussianEstimates)gx);
		done = true;
		for(int j = 0; j <  numbins; j++) {
		    if( !bins[j] ) {
			bins[j] = cgxstack[numfields].hasCollapsed(j);
			done &= bins[j];
		    }
		}
	    } 
		
	    if( resetFlag )
		continue;
	    
	    timeHistogram.addTime(numfields);
	    timeHistogram.repaint();	    
	    
	    //  		if( demoType == 1 ) {
	    //  		    coupledFieldGraphPanel.reset(gx, -numfields);
	    //  		    coupledFieldGraphPanel.repaint();
	    //  		    try{
	    //  			Thread.sleep(delay);
	    //  		    } catch(InterruptedException e) {};
       
	
	    cgx = new CoupledGaussianField(numbins, gx, (GaussianEstimates)gx);
	    //cgx = cgx.respawn2(numbins, null, null);	    
	    //cgx = cgx.respawn2(numbins, gx, (GaussianEstimates)gx);
	    if( demoType == 1 ) {
		coupledFieldGraphPanel.reset(cgx, -numfields);
		coupledFieldGraphPanel.repaint();
	    }
	    
	    for( int i = numfields-1; (i >= 0) && !resetFlag; i-- ) {		   
		cgx = cgxstack[i].applyTo(cgx);
		if(demoType == 1 ) {
		    coupledFieldGraphPanel.setField(cgx);
		    coupledFieldGraphPanel.repaint();
		    try{
			Thread.sleep(delay);
		    } catch(InterruptedException e) {};
		}
	    }

	   
	    if( resetFlag || (cgx == null) || (gx == null) ) continue;

	    if( rx == null )
		rx = new HMField(r, pdf, cgx);
	    else
		rx = ((HMField)rx).respawn(r, pdf, cgx);
	    if( rx2 == null )
		rx2 = new HMField(r, pdf, gx);
	    else
		rx2 = ((HMField)rx2).respawn(r, pdf, gx);
	    //rx = new HMField(r, pdf, cgx);
   	    if( demoType <= 2 ) {
		//coupledFieldGraphPanel.setField(cgx); // makes bg blue
   		coupledFieldGraphPanel.reset(rx, +1); // makes bg blue
	    } else {
		coupledFieldGraphPanel.reset(rx2, +1);
	    }
	    coupledFieldGraphPanel.repaint();
	    try{
		Thread.sleep(delay);
	    } catch(InterruptedException e) {};
	
	    
   	    fieldHistogram.addField(rx);
	    fieldDensityGraph.addField(rx2);

   	    histcount++;
   	    if( histcount > histDelay) {
   		histcount = 1;
   		fieldHistogram.computeHistogram();
   		fieldHistogram.repaint();		

		fieldDensityGraph.computeHistogram();
		fieldDensityGraph.repaint();
   	    }
	}
    }    

    public void windowActivated(WindowEvent we) {
    }

    public void windowDeactivated(WindowEvent we) {
    }

    public void windowClosed(WindowEvent we) {
    }

    public void windowClosing(WindowEvent we) {
	stop();
	setVisible(false);
	if(standalone)
	    System.exit(0);
    }

    public void windowDeiconified(WindowEvent we) {
    }

    public void windowIconified(WindowEvent we) {
    }

    public void windowOpened(WindowEvent we) {	
    }

    public static void main(String args[]) {

	standalone = true;
	
	FCFrame c = new FCFrame();

	//c.pack();
	c.setTitle("Field Couplers");
	c.setSize(600,450);
	c.setVisible(true);
	c.init();
	c.start();
    }
}    














