/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

class GorField extends CouplableField
implements GaussianEstimates {


    double w, sigma, sigma2;
    Pdf target;
    double targetMax;

	/**
	 * This constructor accepts a target density for the drift,
	 * and a magnification factor s for the Gaussian increment.
	 * @param r the random number generator
	 * @param targetDensity a Pdf used to compute the drift
	 * @param s the standard deviation for the Gaussian increment
	 */
    GorField(Random r, Pdf targetDensity, double s) {
	w = r.nextGaussian();
	sigma = s;
	sigma2 = sigma * sigma;
	target = targetDensity;
	RealInterval yrange = target.getDensityRange();
	targetMax = yrange.end - yrange.beginning;
    }

    GorField respawn(Random r, Pdf targetDensity, double s) {
	w = r.nextGaussian();
	sigma = s;
	sigma2 = sigma * sigma;
	target = targetDensity;
	RealInterval yrange = target.getDensityRange();
	targetMax = yrange.end - yrange.beginning;
	return this;
    }
    
    public double valueAt(double x) {
	return x + sigma * Math.pow(target.valueAt(x)/targetMax, -0.5) * w;
    }

    public double sigmaAt(double z) {
	return sigma * Math.pow(target.valueAt(z)/targetMax, -0.5);
    }

    public double muAt(double z) {
	return z;
    }

    public RealInterval sigmaRange(RealInterval r) {
	RealInterval tr = target.range(r);
	double tmp = sigma / Math.sqrt(tr.end/targetMax);
	tr.end = sigma / Math.sqrt(tr.beginning/targetMax);
	tr.beginning = tmp;
	return tr;
    }

    public RealInterval muRange(RealInterval r) {
	return r;
    }

    public double bEstimate(RealInterval r, double x, double y) {

	RealInterval tr = target.range(r);

	double t = ( y * y - x * x ) / 2;
	if( y > x ) 
	    t =  ( - tr.beginning * t + tr.end * r.end * (y - x) )/ (sigma2 * targetMax);
	else
	    t = ( - tr.end * t  + tr.beginning * r.beginning * (y - x) )/ (sigma2 * targetMax);

	return Math.exp( +t );
    }

    public double bTildeEstimate(RealInterval r, double x, double y) {

	RealInterval tr = target.range(r);

	double t = ( y * y - x * x ) / 2;
	if( y > x ) 
	    t =  ( - tr.end * t + tr.beginning * r.beginning * (y - x) )/ (sigma2 * targetMax);
	else
	    t = ( - tr.beginning * t  + tr.end * r.end * (y - x) )/ (sigma2 * targetMax);

	return Math.exp( +t );
    }

    public double kernelDensity(double x, double y) {
	double t = y - x;
	return Math.sqrt( target.valueAt(x) ) *
	    Math.exp( - t * t * target.valueAt(x)/(targetMax * 2 * sigma2));
    }

    public double kernelMean(double x) {
	return x;
    }

    public double newVariableAt(Random r, double x) {
	double w = r.nextGaussian();
	return x + sigma * Math.pow(target.valueAt(x)/targetMax, -0.5) * w;
    }

    public double kernelVariance(double x) {
	return sigma2 * targetMax /target.valueAt(x);
    }

    public double maxSigma(double z0, double eps) {
	RealInterval r = target.range(new RealInterval(z0 - eps, z0 + eps));
	return sigma * Math.pow(r.beginning/targetMax, -0.5);
    }

    public double maxMu(double z0, double eps) {
	return eps;
    }

    public double centerMu(double z0) {
	return z0;
    }

}
