/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class acts as a Hastings-Metropolis machine. It is given a random
 * number generator, a target probability density and a proposal random
 * field and returns the corresponding standard Hastings Metropolis random
 * field. It should be overridden by more specialized HM random fields.
 * @author L.A. Breyer
 * @version 1.0
 */
class HMField extends RandomField {

    double xi;
    Pdf target;
    CouplableField proposal;

	/**
	 * This constructor sets the target density f, the proposal random
	 * field p and the uniform random number used in the Hastings
	 * Metropolis machine. Since typically p is constructed from f also,
	 * it will be usual to override this constructor.
	 * @param r the Random number generator used for accepting/rejecting
	 * @param f the target probability density (unnormalized)
	 * @param p the proposal random field.
	 */
    HMField(Random r, Pdf f, CouplableField p) {
	target = f;
	proposal = p;
	xi = r.nextDouble();
    }

    HMField respawn(Random r, Pdf f, CouplableField p) {
	target = f;
	proposal = p;
	xi = r.nextDouble();
	return this;
    }

	/**
	 * This method returns the value at x of the Hastings Metropolis
	 * random field z goesto X(z).
	 * @param x
	 * @return X(x)
	 */
    public double valueAt(double x) {
	double y = proposal.valueAt(x);
	if( target.valueAt(y) * proposal.kernelDensity(y,x) >
	    xi * target.valueAt(x) * proposal.kernelDensity(x,y) )
	    return y;
	else
	    return x;
    }

	// beta is inverse temperature
	// full MH requires the field to be hot too.
    public double hotValueAt(double x, double beta) {
	double y = proposal.valueAt(x);
	double d = target.valueAt(x) * proposal.kernelDensity(x,y);
	if( d == 0 )
	    return y;
	else if( Math.pow( target.valueAt(y) / target.valueAt(x), beta )
		 * ( proposal.kernelDensity(y,x) /proposal.kernelDensity(x,y) )
		 > xi )
	    return y;
	else
	    return x;
    }
    
    public double newVariableAt(Random r, double x) {
	double xi = r.nextDouble();
	double y = proposal.valueAt(x);
	if( target.valueAt(y) * proposal.kernelDensity(y,x) >
	    xi * target.valueAt(x) * proposal.kernelDensity(x,y) )
	    return y;
	else
	    return x;	
    }
}


