/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class acts as a Hastings-Metropolis machine for simulated annealing.
 * It is given a random
 * number generator, a target probability density and a proposal random
 * field, together with a temperature,
 * and returns the corresponding standard Hastings Metropolis random
 * field.
 * @author L.A. Breyer
 * @version 1.0
 */
class HotHMField extends RandomField {

    double xi;
    Pdf target;
    CouplableField proposal;
    double beta;

	/**
	 * This constructor sets the target density f, the proposal random
	 * field p and the uniform random number used in the Hastings
	 * Metropolis machine. Since typically p is constructed from f also,
	 * it will be usual to override this constructor.
	 * @param r the Random number generator used for accepting/rejecting
	 * @param f the target probability density (unnormalized)
	 * @param p the proposal random field.
	 * @param b the inverse of the temperature: b = 1/T.
	 */
    HotHMField(Random r, Pdf f, CouplableField p, double b) {
	target = f;
	proposal = p;
	xi = r.nextDouble();
	beta = b;
    }

	/**
	 * This method returns the value at x of the Hastings Metropolis
	 * random field z goesto X(z).
	 * @param x
	 * @return X(x)
	 */
    public double valueAt(double x) {
	double y = proposal.valueAt(x);
	if( Math.log(target.valueAt(y) * proposal.kernelDensity(y,x)) >
	    beta * Math.log(xi) + Math.log(target.valueAt(x) * proposal.kernelDensity(x,y)) )
	    return y;
	else
	    return x;
    }

    public double newVariableAt(Random r, double x) {
	double xi = r.nextDouble();
	double y = proposal.valueAt(x);
	if( Math.log(target.valueAt(y) * proposal.kernelDensity(y,x)) >
	    beta * Math.log(xi) + Math.log(target.valueAt(x) * proposal.kernelDensity(x,y)) )
	    return y;
	else
	    return x;	
    }
}
