/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.util.Random;

/**
 * This class implements an (unnormalized) probability density on the
 * real line.
 * @author L.A. Breyer
 * @version 1.0
 */
abstract class Pdf extends RealFunction {

	/**
	 * This method returns the gradient of the logarithm of the density,
	 * evaluated at a point.
	 * @param x the point at which to evaluate
	 * @return the value of the gradient of the logarithm at x.
	 */
    abstract public double gradLogAt(double x);

	/**
	 * This method returns an estimate of the range of the gradient
	 * of log density evaluated over the interval xrange.
	 * @param xrange
	 * @return range of grad log density over interval xrange
	 */
    abstract public RealInterval gradLogRange(RealInterval xrange);
    
	/**
	 * This method returns a RealInterval representing the range of
	 * the density over the interval (0,1).
	 * @return the range of the density.
	 * @see RealInterval
	 */
    abstract public RealInterval getDensityRange();

	/**
	 * This method returns a RealInterval representing the range of
	 * the density over the interval xrange.
	 * @param xrange a RealInterval.
	 * @return the range of the density.
	 * @see RealInterval
	 */
    abstract public RealInterval range(RealInterval xrange);
    
	/**
	 * If f denotes the density, this method returns the ratio
	 * f(y)/f(x) in case f(x) > 0, and 1 otherwise.
	 * @param x
	 * @param y
	 * @return f(y)/f(x) or 1 if f(x) == 0
	 */
    public double densityRatio(double x, double y) {
	if( valueAt(x) == 0 )
	    return 1;
	else
	    return valueAt(y)/valueAt(x);
    }

    
	/**
	 * This method returns a random variable generated from the density.
	 * This function needs to be overridden, as it returns 0 by default.
	 * @param r the random number generator
	 * @return a random number generated from the density
	 */
    abstract public double newVariable(Random r);

    abstract public double getMean();

    abstract public double getVariance();

//    abstract public double getNormalizingConstant();
}










