/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.util.Random;

/**
 * This class implements an (unnormalized) probability density on the
 * interval (0,1), generated from a polygonal description.
 * @author L.A. Breyer
 * @version 1.0
 */
final class PolygonalPdf extends Pdf {

    double shapeX[];
    double shapeY[];
    int n;
    RealInterval xRange, yRange;

    boolean nIsEven = false;
    
	/**
	 * This constructor constructs a density on (0,1) from a set of
	 * coordinates given in two arrays.
	 * @param aSize the number of points
	 * @param aShapeX[] the x coordinates of the points (they must all
	 * lie within the interval (0,1)
	 * @param aShapeY[] the y coordinates of the points.
	 */
    PolygonalPdf( int asize, double aShapeX[], double aShapeY[]) {
	n = asize;

	if( (n % 2) == 0)
	    nIsEven = true;
	
	xRange = new RealInterval(0d,0d);
	yRange = new RealInterval(1000d,0d);
		
  	shapeX = new double[n];
  	shapeY = new double[n];
	
  	for(int i=0; i < n; i++) {
  	    shapeX[i] = aShapeX[i];
  	    xRange.beginning = (xRange.beginning > shapeX[i]) ? shapeX[i]
  		: xRange.beginning;
  	    xRange.end = (xRange.end < shapeX[i]) ? shapeX[i] : xRange.end;

  	    shapeY[i] = aShapeY[i];
  	    yRange.beginning = (yRange.beginning > shapeY[i]) ? shapeY[i]
  		: yRange.beginning;
  	    yRange.end = (yRange.end < shapeY[i]) ? shapeY[i] : yRange.end;
	}
    }

    public double getMean() {
	return 0.5;
    }

    public double getVariance() {
	return 0.1;
    }
    
	/**
	 * This method generates a random variable with the given
	 * polygonal density, by rejection sampling with a uniform
	 * envelope on the interval [0,1]. This works as the density
	 * itself is defined (by assumption) on the interval [0,1].
	 * If rejection sampling is too slow, it returns the value -1 instead.
	 * @param r the random number generator
	 * @return a random variable with polygonal density.
	 */
    public double newVariable(Random r) {
	double prop, u, mag;
	
	mag = yRange.end - yRange.beginning;
	for(int i = 0; i < 500; i++) {
	    prop = r.nextDouble();
	    u = r.nextDouble();
	    
//	    if( u * mag <= valueAt(prop) ) density is now bounded by 1
	    if( u <= valueAt(prop) )
		return prop;
	}
	return -1;
    }

    
    public double valueAt(double x) {
 	if( (x < xRange.beginning) || (x > xRange.end) || n < 2 )
 	    return 0;
	
	double v = 0;

	int i = n - 1;
	if( nIsEven ) {
	    if( x >= shapeX[i-1] ) {
		return( (x - shapeX[i-1])*(shapeY[i]-shapeY[i-1])/
		    (shapeX[i] - shapeX[i-1]) + shapeY[i-1] )/ yRange.end;
	    } else
		i--;
	}

	int eps = i/2;
	i = eps;
	while( eps > 0 ) {
	    if( x >= shapeX[i] )
		if( x < shapeX[i+1] )
		    return( (x - shapeX[i])*(shapeY[i+1]-shapeY[i])/
			(shapeX[i+1] - shapeX[i]) + shapeY[i] ) / yRange.end;
		else
		    i += eps;
	    else
		i -= eps;
	    eps /= 2;
	}

	return( (x - shapeX[i])*(shapeY[i+1]-shapeY[i])/
	    (shapeX[i+1] - shapeX[i]) + shapeY[i] )/ yRange.end;		
    }

    public RealInterval getDensityRange() {
	return new RealInterval( 0, 1); //yRange;
    }    

    public double gradLogAt(double x) {
	if( (x < xRange.beginning) || (x > xRange.end) )
	    return (double)0;

	int i=1;
	do {
	    if( x < shapeX[i] )
		break;
	    i++;
	} while( i < n );

	double ydelta = shapeY[i] - shapeY[i-1];
	return ydelta/( ydelta * (x - shapeX[i-1]) +
			shapeY[i-1] * (shapeX[i] - shapeX[i-1]));
    }

    public RealInterval gradLogRange(RealInterval xrange) {
	double bot = 0, top = 0, a, b;

	boolean first = true;
  	for(int i=1; i < n; i++) 
	    if(xrange.beginning <= shapeX[i]) {
		a = Math.max(xrange.beginning, shapeX[i-1]);
		b = Math.min(xrange.end, shapeX[i]);
		
		double ydelta = shapeY[i] - shapeY[i-1];
		double xdelta = shapeX[i] - shapeX[i-1];
		double gloga = ydelta/ (ydelta * (a - shapeX[i-1]) + shapeY[i-1] * xdelta);
		double glogb = ydelta/ (ydelta * (b - shapeX[i-1]) + shapeY[i-1] * xdelta);
		if( first ) {
		    top = Math.max(gloga, glogb);
		    bot = Math.min(gloga, glogb);
		    first = false;
		} else if( gloga < glogb ) {
		    top = Math.max(top, glogb);
		    bot = Math.min(bot, gloga);
		} else {
		    top = Math.max(top, gloga);
		    bot = Math.min(bot, glogb);
		}
		
		if( b == xrange.end )
		    break;
	    }
	return new RealInterval(bot, top);
    }

    public RealInterval range(RealInterval xrange) {
	double bot = 0, top = 0, a, b;

	boolean first = true;
  	for(int i=1; i < n; i++) 
	    if(xrange.beginning <= shapeX[i]) {
		a = Math.max(xrange.beginning, shapeX[i-1]);
		b = Math.min(xrange.end, shapeX[i]);
		
		double ydelta = shapeY[i] - shapeY[i-1];
		double xdelta = shapeX[i] - shapeX[i-1];
		double m = ydelta / xdelta;
		double va = m * (a - shapeX[i-1]) + shapeY[i-1];
		double vb = m * (b - shapeX[i-1]) + shapeY[i-1];
		if( first ) {
		    top = Math.max(va, vb);
		    bot = Math.min(va, vb);
		    first = false;
		} else if( va < vb ) {
		    top = Math.max(top, vb);
		    bot = Math.min(bot, va);
		} else {
		    top = Math.max(top, va);
		    bot = Math.min(bot, vb);
		}
		
		if( b == xrange.end )
		    break;
	    }
	return new RealInterval(bot/yRange.end, top/yRange.end);	
    }
}

