/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.util.Random;

/**
 * This class implements an very mody probability density, build from a
 * superposition of a sine and a cosine wave. 
 * Truncated so as to live on the interval (0,1).
 * The density is proportional to 1 + sin(a pi x) cos( b pi x).
 *
 * @author L.A. Breyer
 * @version 1.0
 */
final class SinCos extends Pdf {

    double api, bpi;

	/**
	 * This constructor creates a mody probability density on the interval (0,1)
	 * with density proportional to  1 + sin( a pi x) cos( b pi x).
	 * @param a
	 * @param b
	 */
    SinCos(double avalue, double bvalue) {
	api = avalue * Math.PI;
	bpi = bvalue * Math.PI;
    }

    	/**
	 * This method returns an SinCos variable, using rejection sampling with a uniform.
	 * @param r a random number generator
	 * @return a SinCos variable
	 */
    public double newVariable(Random r) {
	double prop, u;
	
	for(int i = 0; i < 500; i++) {
	    prop = r.nextDouble();
	    u = r.nextDouble();
	    if( u * 2 <= valueAt(prop) )
		return prop;
	}
	return -1;
    }
 
    public double valueAt(double x) {
	if( (x < 0) || (x > 1) )
	    return 0;
	else
	    return (1 + Math.sin(api * x) * Math.cos(bpi * x))/2;
    }

//    public double densityRatio(double x, double y) {
//	if( (x < 0) || (x > 1) )
//	    return 0;
//	else
//	    return Math.exp( lambda *(x - y));
//   }
    
    public RealInterval getDensityRange() {
	return new RealInterval(0,1);
    }

    public double gradLogAt(double x) {
	double sa = Math.sin(api * x);
	double cb = Math.cos(bpi * x);
	return (api * Math.cos(api * x) * cb - bpi * sa * Math.sin(bpi * x))/(1 + sa * cb);
    }

	// dummy
    public RealInterval gradLogRange(RealInterval r) {
	return new RealInterval(0, 1);
    }

	// dummy
    public RealInterval range(RealInterval xrange) {
	return new RealInterval( 0, 1);
    }

	// dummy
    public double getMean() {
	return 0;
   }

	// dummy
    public double getVariance() {
	return 1;
    }
}
