/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.util.Random;

/**
 * This class implements a trimodal distribution
 * truncated so as to live on the interval (0,1).
 * @author L.A. Breyer
 * @version 1.0
 */
final class Trimodal extends Pdf {

    double sigma2, m1, m2, m3;

	/**
	 * This constructor creates the density with mean m and variance v.
	 * @param m the mean
	 * @param v the variance
	 */
    Trimodal(double s, double a, double b, double c) {
	sigma2 = s * s;
	m1 = a;
	m2 = b;
	m3 = c;
    }

	/**
	 * This method returns a Normal (mean m, variance v) random
	 * variable. Note that this variable is *not* truncated to (0,1).
	 * @param r a random number generator
	 * @return a Gaussian random variable
	 */
	//dummy
    public double newVariable(Random r) {
	return 0;
    }
    
    public double valueAt(double x) {
	if( (x < 0) || (x > 1) )
	    return 0;
	else {
	    double x1 = x - m1;
	    double x2 = x - m2;
	    double x3 = x - m3;
	    return Math.exp( - x1*x1/(2*sigma2)) + Math.exp( - x2*x2/(2*sigma2)) +
		Math.exp( -x3*x3/(2*sigma2));
	}
    }

	// dummy
    public double densityRatio(double x, double y) {
	return 0;
    }
    
    public RealInterval getDensityRange() {
	return new RealInterval(0,1);
    }

    public double gradLogAt(double x) {
	if( (x < 0) || (x > 1) )
	    return 0;
	else {
	    double x1 = x - m1; double e1 = Math.exp( - x1*x1/(2*sigma2));
	    double x2 = x - m2; double e2 = Math.exp( - x2*x2/(2*sigma2));
	    double x3 = x - m3; double e3 = Math.exp( - x3*x3/(2*sigma2));
	    
	    return -( x1 * e1 + x2 * e2 + x3 * e3)/(sigma2 * (e1 + e2 + e3));
	}
    }
	    //dummy
    public RealInterval gradLogRange(RealInterval xrange) {
	return new RealInterval(0,1);
    }

	// dummy
    public RealInterval range(RealInterval r) {

	return new RealInterval( 0, 1);
    }
	// dummy
    public double getMean() {
	return 0;
    }
    
	// dummy
    public double getVariance() {
	return 1;
    }

//     public double getNormalizingConstant() { // for interval (0,1)
// 	double d = 1/1000;
// 	double x = d/2;
// 	double riemann = 0;
// 	for( int i = 0; i < 999; i++) {
// 	    double t = x - mu;
// 	    riemann += Math.exp( - t * t/(2 * sigma2));
// 	    x += d;
// 	}
// 	return riemann * d;	    
//     }
}
