/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class implements an abstract vector valued random field.
 * It can and should be thought of as a function with random values. This
 * is a vector version of the class RandomField, with much the same
 * functionality.
 * @see RandomField
 * @author L.A. Breyer
 * @version 1.0
 */
abstract class VectorRandomField {

    abstract public double[] valueAt(double x[]);
    abstract public int getDimension();
    
	/**
	 * This method returns the value at y of the density (unnormalized)
	 * for the value of the field at x. It should be overriden, as it
	 * returns 0 by default.
	 * @param x[] location in the interval
	 * @param y[] location for possible value of field X(x[])
	 * @return density value at y (unnormalized)
	 * of the random variable X(x)
	 */
    public double kernelDensity(double x[], double y[]) {
	return 0;
    }

	/**
	 * This method returns the mean of the (normalized) density
	 * p(x,y) at x. Returns 0.5 by default.
	 * @param x location in the interval (0,1)
	 */
    public double kernelMean(double x[]) {
	return 0.5;
    }

	/**
	 * This method returns the variance of the (normalized) density
	 * p(x,y) at x. Returns 0.1 by default.
	 * @param x location in the interval (0,1)
	 */
    public double kernelVariance(double x[]) {
	return 0.1;
    }
}

/**
 * This class accepts a VectorRandomField, and represents one of its
 * components as an ordinary real valued RandomField. For this to work,
 * the component index must be given, as well as a vector which serves
 * to fix the values of the remaining components of the original
 * VectorRandomField.
 */
class ComponentRandomField extends RandomField {
    VectorRandomField fullField;
    int index;
    double fixed[];
    
    ComponentRandomField(VectorRandomField vx, int i, double f[]) {
	fullField = vx;

	int n = vx.getDimension();
	fixed = new double[n];
	for( int k = 0; k < n; k++ )
	    fixed[k] = f[k];

	index = i;
    }

    public double valueAt(double x) {
	fixed[index] = x;
	double temp[] = fullField.valueAt(fixed);
	return temp[index];
    }
}
